# /packages/intranet-reporting/www/timesheet-customer-project-xml-xslt.tcl
#
# Copyright (C) 2003-now Project Open Business Solutions S.L. 
#
# All rights reserved. Please check
# https://www.project-open.com/ for licensing details.

ad_page_contract {
} {
    { start_date "" }
    { end_date "" }
    { level_of_detail 2 }
    { truncate_note_length 4000}
    { output_format "html" }
    { project_id:integer 0}
    { task_id:integer 0}
    { company_id:integer 0}
    { user_id:integer 0}
    { cost_center_id:integer 0}
    { invoice_id:integer 0}
    { invoiced_status "" }
    { xslt_template_id 0 }
    { odt_template_id 0 }
    { print_hour_p:multiple ""}
    { chart_type "" }
    { export_absences "" }
}

proc encodeXmlValue {value} {
    regsub -all {&} $value {&amp;} value
    regsub -all {"} $value {&quot;} value; #"
    regsub -all {'} $value {&apos;} value
    regsub -all {<} $value {&lt;} value
    regsub -all {>} $value {&gt;} value
    return $value
}

proc im_reporting_render_odt_template {
   uri_odt_template
   uri_xml
   notation
} { 

    # This function renders a content.xml of an ODT template file against an XML structure.
    # The original XML structure might have transformed before using a XSLT file. 
    
    # @param: uri_odt_template: 	URI of the ODT file that needs to be rendered against a XML file  
    # @param: uri_xml: 			XML representation of a report
    # Example: 
    # <report>
    #	<attributes>
    #		<start_date>2012-10-01</start_date>
    #		...
    #	</attributes>
    #	<user> ...</user>
    #	<lines>...</lines>
    # </report>

    # Create a temporary directory
    set odt_tmp_path [ad_tmpnam]
    ns_log Notice "im_reporting_render_odt_template: tmp_path=$odt_tmp_path"
    ns_mkdir $odt_tmp_path

    # ------------------------------------------------
    # Read the ODT template
    # ------------------------------------------------
    set odt_zip "${odt_tmp_path}.[lindex [split $uri_odt_template "." ] [expr [llength [split $uri_odt_template "." ]]-1]]"
    set odt_content "${odt_tmp_path}/content.xml"

    # XML
    set xml_file "${odt_tmp_path}/xml_file.xml"

    # ------------------------------------------------
    # Create a copy of the ODF template into the temporary directory
    ns_log Notice "im_reporting_render_odt_template: uri_odt_template='$uri_odt_template'"
    ns_cp $uri_odt_template $odt_zip

    # Unzip the odt into the temorary directory
    if {[catch {
	im_exec unzip -d $odt_tmp_path $odt_zip
    } err_msg]} {
	ad_return_complaint 1  [lang::message::lookup "" intranet-reporting.UnableToUnzip "Unable to unzip ODT template. Please verify if you have choosen a file of type: ODT"]
        return
    }

    # Create a copy of the XML template into the temporary dir
    ns_log Notice "im_reporting_render_odt_template: uri_xml='$uri_xml'"
    ns_cp $uri_xml $xml_file

    # --------------------------------------------------------------------
    # Handle 'Report XML': 
    # Get nodes for attributes, company and user 
    # Write lines & create tcl list of "lines" 
    # --------------------------------------------------------------------

    set file [open $xml_file]
    fconfigure $file -encoding "utf-8"
    set xml_content [read $file]
    close $file

    set xml_doc [dom parse $xml_content]
    set root [$xml_doc documentElement]
    set xml_attribute_nodes [$root selectNodes "//attributes/*"]
    set xml_company_nodes [$root selectNodes "//company/*"]
    set xml_user_nodes [$root selectNodes "//user/*"]

    # Create list of "lines" (@@ records)
	set line_list [list]
	set xml_line_nodes [$root selectNodes "//lines/*"]
    foreach line_node $xml_line_nodes {
		lappend line_list [lindex [$line_node asList] 1]
    }

    # How many columns in line table?
    # set no_cols [expr [llength [lindex $line_list 0]]/2]

    # ------------------------------------------------
    # Read the content.xml file of the ODT template 
    # ------------------------------------------------

    set file [open $odt_content]
    fconfigure $file -encoding "utf-8"
    set odt_template_content [read $file]
    close $file

    # -------------------------------------------------------------------
    # HANDLE @@ VARS 
    # Search the <row> ...<cell>..</cell>.. </row> line in a table 
    # representing the part of the template that needs to
    # be repeated for every record ("line").
    # -------------------------------------------------------------------

    # Get the list of all "tables" in the document
    set odt_doc [dom parse $odt_template_content]
    set root [$odt_doc documentElement]
    set odt_table_nodes [$root selectNodes "//table:table"]

    # Search for the table that contains "@@"
    set odt_template_table_node ""
    foreach table_node $odt_table_nodes {
	set table_as_list [$table_node asList]
	if {[regexp {@@} $table_as_list match]} { set odt_template_table_node $table_node }
    }

    # Deal with the the situation that we didn't find the line
    if {"" == $odt_template_table_node} {
	ad_return_complaint 1 "
                        <b>Didn't find table including '@@ vars'</b>:<br>
                        We have found a valid OOoo template at '$uri_odt_template'.
                        However, this template does not include a table with the value
                        above.
                "
	ad_script_abort
    }
	
    # Search for the 2nd table:table-row tag (the first one contains the header) 
    set odt_table_rows_nodes [$odt_template_table_node selectNodes "//table:table-row"]
    set odt_template_double_vars_row_node ""
    set odt_template_formular_row_node ""

    foreach row_node $odt_table_rows_nodes {
	set row_as_list [$row_node asList]
	ns_log Notice "timesheet-customer-project-xml-xslt:: row_as_list: $row_as_list"
	ns_log Notice "timesheet-customer-project-xml-xslt:: --------------------------"
	# Looking for the @@ row ... 
	if {[regexp {@@} $row_as_list match]} { set odt_template_double_vars_row_node $row_node }
	# Looking for a formular 
	if { [string first "ooow:sum" [string tolower $row_as_list]] != -1 } {
	    ns_log Notice "timesheet-customer-project-xml-xslt:: Found formular"
	    set odt_template_formular_row_node $row_node 
	}		
    }

    if {"" == $odt_template_double_vars_row_node} {
	ad_return_complaint 1 "
                        <b>Didn't find row including '@@ vars'</b>:<br>
                        We have found a valid OOoo template at '$uri_odt_template'. However, this template does not include a table with a row taht contains @@ placeholders."
	ad_script_abort
    } 

    # -------------------------------------------------------------------
    # 'odt_template_double_vars_row_node' now contains the row withe the @@ vars 
    # -------------------------------------------------------------------

    # Get the parentNode: <table:table ..>
    set parent_node [$odt_template_double_vars_row_node parentNode]

    # Store the node as a list so that we can clone it
    set row_mold_list [$odt_template_double_vars_row_node asList]

    # -------------------------------------------------------------------
    # Now creating for each record found in the XML report a node and 
    # store them in a list. Later we reserve the list to keep the order
    # -------------------------------------------------------------------
    set ctr_new_nodes_created 0
    set list_new_nodes [list]
    
    foreach line $line_list {
	# Clone node content
        set new_node $row_mold_list
	
        # Replace vars - expecting a well formed key/value list ...
        foreach {i j} $line {
	    # ns_log Notice "timesheet-customer-project-xml-xslt:: $row_mold_list ::: $i $j/[list $j] <br><br>"
	    regsub -all "@@$i@@" $new_node [list [string map {& \\&} $j]] new_node
        }
		
        # Append new node at the end of parent node -> works for ODT but not for CALC
        # set new_node [$parent_node appendFromList $new_node_list]
	lappend list_new_nodes $new_node
	
	# Count lines 
	incr ctr_new_nodes_created
	}

    # -------------------------------------------------------------------
    # Now adjusting any formular 
    # -------------------------------------------------------------------
    set has_attr_form_p 0

    if { "" != $odt_template_formular_row_node } {
	foreach cell_node [$odt_template_formular_row_node childNodes] {
	    set has_attr_form_p [$cell_node hasAttribute "table:formula"]
	    if { $has_attr_form_p } {
		# example of value expected: ooow:sum<b2:b2>
		set formular [$cell_node getAttribute "table:formula"]
		
		# Get the coordinate-pair
		regexp -nocase {<(.*?)>} $formular coordinate_pair
		
		# Get integer-part of second part of coordinate-pair (till)
		regexp -nocase {[0-9]+} [lindex [split $coordinate_pair ":"] 1] coordinate_int_orig
		
		# Update target cell coordinate
		if {[catch {
		    # Calculate new target cell coordinate
		    set coordinate_int_new [expr {$coordinate_int_orig + $ctr_new_nodes_created -1}]
		    ns_log NOTICE "timesheet-customer-project-xml-xslt:: New value of coordinate_int: $coordinate_int_new"
		    set cmd "set target_coordinate \[string map \{$coordinate_int_orig $coordinate_int_new\} \"[lindex [split $coordinate_pair ":"] 1]\"\]"
		    eval $cmd
		    
		    set formular "ooow:sum[lindex [split $coordinate_pair ":"] 0]:$target_coordinate"
		    $cell_node setAttribute "table:formula" $formular
		    
		} err_msg]} {				
		    ns_log Notice "timesheet-customer-project-xml-xslt:: Error updating forumular: $err_msg"
		}
	    }
	}
    }
    
    set formular_row_list [$odt_template_formular_row_node asList]
    
    # Remove the line with @@ placeholders
    $odt_template_double_vars_row_node delete
    $odt_template_formular_row_node delete 
    
    # reverse list to keep order 
    lreverse $list_new_nodes
    
    # Now add the newly created rows 
    foreach row $list_new_nodes {
	$parent_node appendFromList $row
    }
    
    $parent_node appendFromList $formular_row_list

    # ------------------------------------------------
    # HANDLE @ VARS 
    # Now replace single '@' -> regular vars
    # ------------------------------------------------

    # Render entire ODT to replace vars
    set content_xml [$root asXML -indent none]
	
    # ToDo: Optimize
    foreach attribute_node $xml_attribute_nodes {
	if { "" != [[$attribute_node selectNodes "."] text]  } {
	    regsub -all "@[$attribute_node nodeName]@" $content_xml "[[$attribute_node selectNodes "."] text]" content_xml
	} else {
	    regsub -all "@[$attribute_node nodeName]@" $content_xml "" content_xml
	}
    }	
    
    foreach company_node $xml_company_nodes {
        if { "" != [[$company_node selectNodes "."] text]  } {
            regsub -all "@[$company_node nodeName]@" $content_xml "[[$company_node selectNodes "."] text]" content_xml
        } else {
            regsub -all "@[$company_node nodeName]@" $content_xml "" content_xml
        }
    }

    foreach user_node $xml_user_nodes {
        if { "" != [[$user_node selectNodes "."] text]  } {
            regsub -all "@[$user_node nodeName]@" $content_xml "[[$user_node selectNodes "."] text]" content_xml
        } else {
            regsub -all "@[$user_node nodeName]@" $content_xml "" content_xml
        }
    }
    # /ToDo

    # Save the content to a file.
    set file [open $odt_content w]
    fconfigure $file -encoding "utf-8"
    puts $file $content_xml
    flush $file
    close $file

    # The zip -j command replaces the specified file in the OO zipfile
    ns_log Notice "intranet-reporting-procs.tcl: before zipping"
    im_exec zip -j $odt_zip $odt_content
    return $odt_zip
}

# ------------------------------------------------------------
# Security & Defaults 
# ------------------------------------------------------------

# Label: Provides the security context for this report
# because it identifies unquely the report's Menu and
# its permissions.
set menu_label "reporting-timesheet-customer-project-xml-xslt"
set current_user_id [auth::require_login]
set use_project_name_p [parameter::get_from_package_key -package_key intranet-reporting -parameter "UseProjectNameInsteadOfProjectNr" -default 0]
set xml_output ""

# Default User = Current User, to reduce performance overhead
if {"" == $start_date && "" == $end_date && 0 == $project_id && 0 == $company_id && 0 == $user_id} { 
    set user_id $current_user_id 
}

set read_p [db_string report_perms "
	select	im_object_permission_p(m.menu_id, :current_user_id, 'read')
	from	im_menus m
	where	m.label = :menu_label
" -default 'f']

# Has the current user the right to edit all timesheet information?
set edit_timesheet_p [im_permission $current_user_id "add_hours_all"]

# ToDo: remove after V3.5: compatibility with old privilege
if {[im_permission $current_user_id "edit_hours_all"]} {set edit_timesheet_p 1 }

set view_hours_all_p [im_permission $current_user_id "view_hours_all"]

if {!$view_hours_all_p} { set user_id $current_user_id }

# If project_id and task_id are set and equal, exclude task_id from sql   
if {0 != $task_id && "" != $task_id && 0 != $project_id && "" != $project_id && $project_id == $task_id} {
    set task_id 0
}

set im_survey_installed_p [db_string get_package_id "select package_id from apm_packages where package_key = 'intranet-sencha'" -default 0]

if { "chart"==$output_format && 0==$im_survey_installed_p } {
    ad_return_complaint 1 "ExtJS libs required, please checkout and install package intranet-sencha"
}

# ------------------------------------------------------------
# Constants

set number_format "999,990.99"
set invoice_template_base_path [im_parameter -package_id [im_package_invoices_id] InvoiceTemplatePathUnix "" "/tmp/templates/"]
set path_saxon  [parameter::get -package_id [apm_package_id_from_key intranet-openoffice] -parameter "PathSaxonParser" -default ""]
if { "" == $path_saxon  } {
    ad_return_complaint 1  [lang::message::lookup "" intranet-openoffice.PathToSaxonParserNotFound "Path to Saxon Parser not defined. Please check Parameter 'PathSaxonParser' of package 'intranet-openoffice'"]
}

# ------------------------------------------------------------
# Validation 

if { "template" == $output_format && "" == $odt_template_id } {
	ad_return_complaint 1  [lang::message::lookup "" intranet-reporting.ChooseTemplate "Please choose a template"]
}

if {"t" ne $read_p } {
    ad_return_complaint 1 "
    [lang::message::lookup "" intranet-reporting.You_dont_have_permissions "You don't have the necessary permissions to view this page"]"
    return
}

# Check that Start & End-Date have correct format
if {"" != $start_date && ![regexp {^[0-9][0-9][0-9][0-9]\-[0-9][0-9]\-[0-9][0-9]$} $start_date]} {
    ad_return_complaint 1 "Start Date doesn't have the right format.<br>
    Current value: '$start_date'<br>
    Expected format: 'YYYY-MM-DD'"
}

if {"" != $end_date && ![regexp {^[0-9][0-9][0-9][0-9]\-[0-9][0-9]\-[0-9][0-9]$} $end_date]} {
    ad_return_complaint 1 "End Date doesn't have the right format.<br>
    Current value: '$end_date'<br>
    Expected format: 'YYYY-MM-DD'"
}

set page_title "Timesheet Customer & Project - Extended Version \[BETA\]"
set context_bar [im_context_bar $page_title]
set context ""

switch output_format {
    html { set page_title_adjusted [lang::message::lookup "" intranet-reporting.OutputFormatHTML "HTML"] }
    chart { set page_title_adjusted [lang::message::lookup "" intranet-reporting.OutputFormatChart "Chart"] }
    default { set page_title_adjusted $page_title }
}

# ------------------------------------------------------------
# Defaults

set rowclass(0) "roweven"
set rowclass(1) "rowodd"

set days_in_past 7

db_1row todays_date "
select
	to_char(sysdate::date - :days_in_past::integer, 'YYYY') as todays_year,
	to_char(sysdate::date - :days_in_past::integer, 'MM') as todays_month,
	to_char(sysdate::date - :days_in_past::integer, 'DD') as todays_day
from dual
"

if {"" == $start_date} { 
    set start_date "$todays_year-$todays_month-01"
}

# Maxlevel is 4. Normalize in order to show the right drop-down element
if {$level_of_detail > 5} { set level_of_detail 5 }


db_1row end_date "
select
	to_char(to_date(:start_date, 'YYYY-MM-DD') + 31::integer, 'YYYY') as end_year,
	to_char(to_date(:start_date, 'YYYY-MM-DD') + 31::integer, 'MM') as end_month,
	to_char(to_date(:start_date, 'YYYY-MM-DD') + 31::integer, 'DD') as end_day
from dual
"

if {"" == $end_date} { 
    set end_date "$end_year-$end_month-01"
}


set company_url "/intranet/companies/view?company_id="
set project_url "/intranet/projects/view?project_id="
set user_url "/intranet/users/view?user_id="
set hours_url "/intranet-timesheet2/hours/one"
set this_url [export_vars -base "/intranet-reporting/timesheet-customer-project-xml-xslt" {start_date end_date level_of_detail project_id task_id company_id user_id } ]

# BaseURL for drill-down. Needs company_id, project_id, user_id, level_of_detail
set base_url [export_vars -base "/intranet-reporting/timesheet-customer-project-xml-xslt" {start_date end_date task_id} ]

# ------------------------------------------------------------
# Conditional SQL Where-Clause
#

set criteria [list]
set criteria_absences [list]

if {0 != $company_id && "" != $company_id} {
    lappend criteria "p.company_id = :company_id"
}

if {0 != $user_id && "" != $user_id} {
    lappend criteria "h.user_id = :user_id"
    lappend criteria_absences "cc.user_id = :user_id"
}

if {0 != $cost_center_id && "" != $cost_center_id} {
    set cc_code [db_string cc_code "select cost_center_code from im_cost_centers where cost_center_id = :cost_center_id" -default "Co"]
    set cc_code_len [string length $cc_code]

    lappend criteria "h.user_id in (
		select	e.employee_id
		from	im_employees e
		where	e.department_id in (
			select	cost_center_id
			from	im_cost_centers
			where	substring(cost_center_code, 1, :cc_code_len) = :cc_code
		)
    )"

    lappend criteria_absences "cc.user_id in (
                select  e.employee_id
                from    im_employees e
                where   e.department_id in (
                        select  cost_center_id
                        from    im_cost_centers
                        where   substring(cost_center_code, 1, :cc_code_len) = :cc_code
                )
    )"
}

if {0 != $task_id && "" != $task_id} {
    lappend criteria "h.project_id = :task_id"
}

if {0 != $invoice_id && "" != $invoice_id} {
    lappend criteria "h.invoice_id = :invoice_id"
}

if {"" != $invoiced_status} {
    switch $invoiced_status {
	"invoiced" { lappend criteria "h.invoice_id is not null" }
	"not-invoiced" { lappend criteria "h.invoice_id is null" }
	default { ad_return_complaint 1 "<b>Invalid option for 'invoiced_status': '$invoiced_status'</b>:<br>Only 'invoiced' and 'not-invoiced' are allowed." }
    }
}


# Select project & subprojects
set org_project_id $project_id
if {0 != $project_id && "" != $project_id} {
    lappend criteria "p.project_id in (
	select
		p.project_id
	from
		im_projects p,
		im_projects parent_p
	where
		parent_p.project_id = :project_id
		and p.tree_sortkey between parent_p.tree_sortkey and tree_right(parent_p.tree_sortkey)
		and p.project_status_id not in ([im_project_status_deleted])
    )"
}

if { "" != $print_hour_p } {
	set hour_id_in ""
	foreach hour_id $print_hour_p {
		append hour_id_in "$hour_id," 
	}
	lappend criteria "h.hour_id in ([string range $hour_id_in 0 end-1])"
}

set where_clause [join $criteria " and\n	    "]
if { $where_clause ne "" } {
    set where_clause " and $where_clause"
}

set where_clause_absences [join $criteria_absences " and\n	    "]
if { $where_clause_absences ne "" } {
    set where_clause_absences " and $where_clause_absences"
}


# -- ----------------------------------------------------------
#    Set absence array
# -- ----------------------------------------------------------

set absence_list [list]

if { "" != $export_absences } {
    set sql "
    select 
    	 cc.user_id as active_employee_id,
	 im_initials_from_user_id(cc.user_id) as user_initials
    from 
	 cc_users cc,
         acs_rels r
    where 
         r.object_id_one = 463 and 
         r.object_id_two =cc.party_id and
         r.rel_type = 'membership_rel' and
         cc.member_state = 'approved'
         $where_clause_absences
    " 
    db_foreach active_employee_id $sql {
	set inner_sql "select * from im_absences_get_absences_for_user_duration(:active_employee_id, :start_date, :end_date, null) AS (absence_date date, absence_type_id int, absence_id int, duration_days numeric)"
	db_foreach r $inner_sql {
	    
	    # Building hash key out of julian date and user_id helps us to sort the arr/list
	    # Please note: To achieve same sort order in all absence-type-groups all user_id's woudl need to have the same number of digits 
	    set hash_key "${absence_type_id}.${active_employee_id}[dt_ansi_to_julian_single_arg $absence_date]"
	    
	    # For this report max absence for a single day is '1'  
	    if { [info exists absence_hash($hash_key)] } {
		# Add only when absence duration is less than one day
		if { [lindex $absence_hash($hash_key) 0] < 1 } { 
		    set absence_hash($hash_key) [list [expr {[lindex $absence_hash($hash_key) 0] + $duration_days}] "-1" $absence_date $active_employee_id $user_initials]
		    if { [lindex $absence_hash($hash_key) 0] >= 1 } {
			set absence_hash($hash_key) [list 1 "-1"]
		    }
		}
	    } else {
		set absence_hash($hash_key) [list $duration_days $absence_type_id $absence_date $active_employee_id $user_initials]
	    }
	}
    }
    
    # We convert to list for tcl < 8.6 
    set x [list]
    foreach {k v} [array get absence_hash] { lappend x [list $k $v] }
    set absence_list [lsort -real -index 0 $x]
    # ns_return 1 text/html $absence_list
}

# -- ----------------------------------------------------------
#    Define the report - SQL, counters, headers and footers 
# -- ----------------------------------------------------------

set sql "
SELECT 
	h.note,
	h.internal_note,
	h.hour_id,
	to_char(h.day, 'YYYY-MM-DD') as date_pretty,
	to_char(h.day, 'J') as julian_date,
	to_char(h.day, 'J')::integer - to_char(to_date( :start_date , 'YYYY-MM-DD'), 'J')::integer as date_diff,
	to_char(coalesce(h.hours,0), :number_format ) as hours,
	to_char(h.billing_rate, :number_format ) || '&nbsp;' || co.currency as billing_rate,
	u.user_id,
	im_name_from_user_id(u.user_id) as user_name,
	im_initials_from_user_id(u.user_id) as user_initials,
	main_p.project_id,
	main_p.project_nr,
	main_p.project_name,
	p.project_id as sub_project_id,
	p.project_nr as sub_project_nr,
	p.project_name as sub_project_name,
	c.company_id,
	c.company_path as company_nr,
	c.company_name,
	c.company_id || '-' || main_p.project_id as company_project_id,
	c.company_id || '-' || main_p.project_id || '-' || p.project_id as company_project_sub_id,
	c.company_id || '-' || main_p.project_id || '-' || p.project_id || '-' || u.user_id as company_project_sub_user_id
FROM 
	im_hours h,
	im_projects p,
	im_projects main_p,
	im_companies c,
	users u, 
	im_costs co
WHERE 
	h.cost_id = co.cost_id 
	and h.project_id = p.project_id
	and main_p.project_status_id not in ([im_project_status_deleted])
	and h.user_id = u.user_id
	and main_p.tree_sortkey = tree_root_key(p.tree_sortkey)
	and h.day >= to_timestamp( :start_date , 'YYYY-MM-DD' )
	and h.day < to_timestamp( :end_date , 'YYYY-MM-DD' )
	and main_p.company_id = c.company_id
	$where_clause

ORDER BY 
	company_path,
	main_p.project_nr,
	project_nr,
	user_name,
	project_nr,
	day
"

set report_def [list \
	group_by company_id \
	header {
		""
		"\#colspan=98 <a href=$base_url&company_id=$company_id&level_of_detail=4 
		target=_blank><img src=/intranet/images/plus_9.gif border=0></a> 
		<b><a href=$company_url$company_id>$company_name</a></b>"
	} \
	content [list  \
		group_by company_project_id \
		header {
			""
			$company_nr 
			"\#colspan=99 <a href=$base_url&project_id=$project_id&level_of_detail=4 
			target=_blank><img src=/intranet/images/plus_9.gif border=0></a>
			<b><a href=$project_url$project_id>$project_name</a></b>"
		} \
		content [list \
			group_by company_project_sub_id \
			header {
				""
				$company_nr 
				$project_nr 
				"\#colspan=99 <a href=$base_url&project_id=$sub_project_id&level_of_detail=5
				target=_blank><img src=/intranet/images/plus_9.gif border=0></a>
				<b><a href=$project_url$sub_project_id>$sub_project_name</a></b>"
			} \
			content [list \
				group_by company_project_sub_user_id \
				header {
					""
					$company_nr 
					$project_nr 
					$sub_project_nr 
					"\#colspan=99 <a href=$base_url&project_id=$sub_project_id&user_id=$user_id&level_of_detail=5
					target=_blank><img src=/intranet/images/plus_9.gif border=0></a>
					<b><a href=$user_url$user_id>$user_name</a></b>"
				} \
				content [list \
					header {
						"<input type='checkbox' name='print_hour_p' value='$hour_id'>" 
						$company_nr
						$project_nr
						$sub_project_nr
						$user_initials
						"<nobr>$date_pretty</nobr>"
						$hours_link
						$billing_rate
						"<nobr>$note</nobr>"
					} \
					content {} \
				] \
				footer {
					""
					$company_nr 
					$project_nr 
					$sub_project_nr 
					$user_initials
					""
					"<i>$hours_user_subtotal</i>"
					""
					""
				} \
			] \
			footer {
				""
				$company_nr
				$project_nr
				$sub_project_nr
				""
				""
				"<i>$hours_project_sub_subtotal</i>"
				""
				""
			} \
		] \
		footer {
			""			
			$company_nr
			$project_nr
			""
			""
			""
			"<b>$hours_project_subtotal</b>"
			""
			""
		} \
	] \
	footer {"" "" "" "" "" "" "" "" ""} \
]


# Global header/footer
set header0 {"Print" "Customer" "Project" "Subproject" "User" "Date" Hours Rate Note}
set footer0 {"" "" "" "" "" "" "" "" ""}

# If user is not allowed to see internal rates we remove 'rate' items from record 
if { ![im_permission $current_user_id "fi_view_internal_rates"] } {
    set report_def [string map {\$billing_rate ""} $report_def] 
    set header0 [string map {"Rate" ""} $header0]
}

set hours_user_counter [list \
	pretty_name Hours \
	var hours_user_subtotal \
	reset \$company_project_sub_user_id \
	expr \$hours
]

set hours_project_sub_counter [list \
	pretty_name Hours \
	var hours_project_sub_subtotal \
	reset \$company_project_sub_id \
	expr \$hours
]

set hours_project_counter [list \
	pretty_name Hours \
	var hours_project_subtotal \
	reset \$company_project_id \
	expr \$hours
]

set hours_customer_counter [list \
	pretty_name Hours \
	var hours_customer_subtotal \
	reset \$company_id \
	expr \$hours
]

set counters [list \
	$hours_user_counter \
	$hours_project_sub_counter \
	$hours_project_counter \
	$hours_customer_counter \
]


# ------------------------------------------------------------
# Constants
#

set start_years {2010 2010 2011 2011 2012 2012 2013 2013 2014 2014 2015 2015 2016 2016 2017 2017}
set start_months {01 Jan 02 Feb 03 Mar 04 Apr 05 May 06 Jun 07 Jul 08 Aug 09 Sep 10 Oct 11 Nov 12 Dec}
set start_weeks {01 1 02 2 03 3 04 4 05 5 06 6 07 7 08 8 09 9 10 10 11 11 12 12 13 13 14 14 15 15 16 16 17 17 18 18 19 19 20 20 21 21 22 22 23 23 24 24 25 25 26 26 27 27 28 28 29 29 30 30 31 31 32 32 33 33 34 34 35 35 36 36 37 37 38 38 39 39 40 40 41 41 42 42 43 43 44 44 45 45 46 46 47 47 48 48 49 49 50 50 51 51 52 52}
set start_days {01 1 02 2 03 3 04 4 05 5 06 6 07 7 08 8 09 9 10 10 11 11 12 12 13 13 14 14 15 15 16 16 17 17 18 18 19 19 20 20 21 21 22 22 23 23 24 24 25 25 26 26 27 27 28 28 29 29 30 30 31 31}
set levels {1 "Customer Only" 2 "Customer+Project" 3 "Customer+Project+Sub" 4 "Customer+Project+Sub+User" 5 "All Details"} 
set truncate_note_options {4000 "Full Length" 80 "Standard (80)" 20 "Short (20)"} 

set invoiced_status_options [list "" "All" "invoiced" "Only invoiced hours" "not-invoiced" "Only not invoiced hours"]

# ------------------------------------------------------------
# Start formatting the page
#

set report_options_html ""
if {$level_of_detail > 3} {
    append report_options_html "
	<tr>
	  <td class=form-label>Size of Note Field</td>
	  <td class=form-widget>
	    [im_select -translate_p 0 truncate_note_length $truncate_note_options $truncate_note_length]
	  </td>
	</tr>
    "
}

if {[info exists task_id]} {
    append report_options_html "
	<tr>
	  <td class=form-label></td>
	  <td class=form-widget>
	    <input type=hidden name=task_id value=\"$task_id\">
	  </td>
	</tr>
    "
}



# ------------------------------------------------------------
# Start Formatting the HTML Page Contents

# Write out HTTP header, considering CSV/MS-Excel formatting
switch $output_format {
    html - csv {
	   im_report_write_http_headers -output_format $output_format -report_name "timesheet-customer-project-xml-xslt"
    }
    chart {
	   im_report_write_http_headers -output_format "html"
	   ns_write "<link rel='stylesheet' href='/intranet-sencha/css/ext-all.css'  type='text/css' media='screen'>"
	   ns_write "<script type='text/javascript' src='/intranet-sencha/js/ext-all-debug-w-comments.js'></script>"
    }
}

set project_id $org_project_id

set sidebar_html "
	[im_header $page_title]
	[im_navbar reporting]
	<form method='POST'>
		<div id=\"slave\">
			<div id=\"slave_content\">
				 <div class=\"filter-list\">
				  	<div class=\"filter\">
						<div class=\"filter-block\">
							[export_vars -form {invoice_id}]
								<table border=0 cellspacing=1 cellpadding=1>
									<tr valign=top><td>
											<table border=0 cellspacing=1 cellpadding=1>
													<tr>
															<td class=form-label>Level of Details</td>
															<td class=form-widget>
															 	 [im_select -translate_p 0 level_of_detail $levels $level_of_detail]
															 </td>
													</tr>
													<tr>
														<td class=form-label>Start Date</td>
														<td class=form-widget>
															<input type=textfield name=start_date value=$start_date>
														</td>
													</tr>
													<tr>
														 <td class=form-label>End Date</td>
														 		  <td class=form-widget>
		    <input type=textfield name=end_date value=$end_date>
		  </td>
		</tr>
		<tr>
		  <td class=form-label>Customer</td>
		  <td class=form-widget>
		    [im_company_select -include_empty_name [lang::message::lookup "" intranet-core.All "All"] company_id $company_id]
		  </td>
		</tr>
		<tr>
		  <td class=form-label>Project</td>
		  <td class=form-widget>
		    [im_project_select -include_empty_p 1 -exclude_subprojects_p 0 -include_empty_name [lang::message::lookup "" intranet-core.All "All"] project_id $project_id]
		  </td>
		</tr>

"

	if {$view_hours_all_p} {
	    append sidebar_html "
		<tr>
		  <td class=form-label>User's Department</td>
		  <td class=form-widget>
		    [im_cost_center_select -include_empty 1 -include_empty_name [lang::message::lookup "" intranet-core.All "All"] -department_only_p 1 cost_center_id $cost_center_id]
		  </td>
		</tr>
		<tr>
		  <td class=form-label>User</td>
		  <td class=form-widget>
		    [im_user_select -include_empty_p 1 -group_id [list [im_employee_group_id] [im_freelance_group_id]] -include_empty_name [lang::message::lookup "" intranet-core.All "All"] user_id $user_id] 
		</td>
		</tr>
		<tr>
		  <td class=form-label>
			[lang::message::lookup "" intranet-core.Invoiced_Status "Invoiced Status"]
		  </td>
		  <td class=form-widget>
		    [im_select invoiced_status $invoiced_status_options $invoiced_status]
		</td>
		</tr>
	    "
	}


if { ![info exists locale] || "" == $locale} { set locale [lang::user::locale] }
set html_checked ""
set excel_checked ""
set csv_checked ""
set xml_checked ""
set template_checked ""
set chart_checked ""
set chart_type_pie_customer_checked ""
set chart_type_pie_project_type_checked "" 
set export_absences_checked ""

set template_row_visibility "row_hidden"
set chart_row_visibility "row_hidden"
set absence_row_visibility "row_hidden"
switch $output_format {
    html - printer { 
	set html_checked "checked" 
        set absence_row_visibility "row_visible"
    }
    excel { set excel_checked "checked" }
    csv { 
	set csv_checked "checked" 
        set absence_row_visibility "row_visible"
    }
    xml { set xml_checked "checked" }
    template { 
	set template_checked "checked" 
	set template_row_visibility "row_visible"
    }
    chart { 
	set chart_checked "checked"
	set chart_row_visibility "row_visible"
    }
}

if { "" != $export_absences } { set export_absences_checked "checked" }

switch chart_type {
    chart_type_pie_customer { set chart_type_pie_customer_checked "checked" }
    chart_type_pie_project_type { set chart_type_pie_project_type_checked "checked" }
}

append sidebar_html "
		$report_options_html
		<tr>
		  <td class=form-label>[lang::message::lookup "" intranet-reporting.Format "Format"]</td>
		  <td class=form-widget>
		    	    	<input name='output_format' type=radio value='html' $html_checked onclick='handleClick(this);'>HTML<br>
	   			<input name='output_format' type=radio value='csv' $csv_checked onclick='handleClick(this);'>CSV<br>
	   			<input name='output_format' type=radio value='xml' $xml_checked onclick='handleClick(this);'>XML<br>
	   			<input name='output_format' type=radio value='template' $template_checked onclick='handleClick(this);'>[lang::message::lookup "" intranet-reporting.Template "Template"]<br/>
	   			<input name='output_format' type=radio value='chart' $chart_checked onclick='handleClick(this);'>[lang::message::lookup "" intranet-reporting.Chart "Chart"]
		  </td>
		</tr>
    <tr class='$absence_row_visibility' id='absences'>
          <td class=form-label> [lang::message::lookup "" intranet-reporting.ExportAbsences "Show Absences"]</td>
          <td class=form-widget><input type='checkbox' name='export_absences' $export_absences_checked></td>
    </tr>
    <tr class='$template_row_visibility' id='xslt_template'>
          <td class=form-label> [lang::message::lookup "" intranet-reporting.XSLTFilter "XSLT Filter"]</td>
          <td class=form-widget>[im_category_select -include_empty_p 1 -include_empty_name [lang::message::lookup "" intranet-core.Please_Select "Please select"] -translate_p 0 "Intranet Cost Template" "xslt_template_id" $xslt_template_id]</td>
    </tr>
    <tr class='$template_row_visibility' id='odt_template'>
          <td class=form-label> [lang::message::lookup "" intranet-reporting.ODTTemplate "ODT Template"]</td>
          <td class=form-widget>[im_category_select -include_empty_p 1  -include_empty_name [lang::message::lookup "" intranet-core.Please_Select "Please select"] -translate_p 0 "Intranet Cost Template" "odt_template_id" $odt_template_id]</td>
    </tr>	
    <tr class='$chart_row_visibility' id='chart_type'>
          <td class=form-label> [lang::message::lookup "" intranet-core.ChartType "Chart Type"]</td>
          <td class=form-widget>
<select name='chart_type'>
<option value='chart_type_pie_customer' $chart_type_pie_customer_checked> [lang::message::lookup "" intranet-reporting.ChartPieCustomer "Pie - Customer"]</option>
<option value='chart_type_pie_project_type' $chart_type_pie_project_type_checked> [lang::message::lookup "" intranet-reporting.ChartPieProjectType "Pie - Project Type"]</option>
</select>
		</td>
    </tr>	
    <tr>
		  <td class=form-label></td>
		  <td class=form-widget><input type=submit value=Submit></td>
		</tr>
		</table>

	</td></tr>
	</table>

	</div> <!-- /filter block --> 
	</div> <!-- /filter -->
    </div> <!-- /filter-list -->
    </div> <!-- /slave-content -->
    </div> <!-- /slave -->


<script type='text/javascript' nonce='[im_csp_nonce]'>
function handleClick(myRadio) \{

	if (\"html\" == myRadio.value) \{
	   document.getElementById('absences').className = document.getElementById('absences').className.replace('row_hidden', 'row_visible');
	   document.getElementById('chart_type').className = document.getElementById('chart_type').className.replace('row_visible', 'row_hidden');
           document.getElementById('odt_template').className = document.getElementById('odt_template').className.replace('row_visible', 'row_hidden');
           document.getElementById('xslt_template').className = document.getElementById('xslt_template').className.replace('row_visible', 'row_hidden');
	\} 

	if (\"csv\" == myRadio.value) \{
	   document.getElementById('absences').className = document.getElementById('absences').className.replace('row_hidden', 'row_visible');
	   document.getElementById('chart_type').className = document.getElementById('chart_type').className.replace('row_visible', 'row_hidden');
           document.getElementById('odt_template').className = document.getElementById('odt_template').className.replace('row_visible', 'row_hidden');
           document.getElementById('xslt_template').className = document.getElementById('xslt_template').className.replace('row_visible', 'row_hidden');
	\};

	if (\"xml\" == myRadio.value) \{
	   document.getElementById('absences').className = document.getElementById('absences').className.replace('row_visible', 'row_hidden');
	   document.getElementById('chart_type').className = document.getElementById('chart_type').className.replace('row_visible', 'row_hidden');
           document.getElementById('odt_template').className = document.getElementById('odt_template').className.replace('row_visible', 'row_hidden');
           document.getElementById('xslt_template').className = document.getElementById('xslt_template').className.replace('row_visible', 'row_hidden');
	\};

	if (\"template\" == myRadio.value) \{
	   document.getElementById('odt_template').className = document.getElementById('odt_template').className.replace('row_hidden', 'row_visible');
	   document.getElementById('xslt_template').className = document.getElementById('xslt_template').className.replace('row_hidden', 'row_visible');
	   document.getElementById('chart_type').className = document.getElementById('chart_type').className.replace('row_visible', 'row_hidden');
	   document.getElementById('absences').className = document.getElementById('absences').className.replace('row_visible', 'row_hidden');
	\} 

	if (\"chart\" == myRadio.value) \{
	   document.getElementById('chart_type').className = document.getElementById('chart_type').className.replace('row_hidden', 'row_visible');
	   document.getElementById('odt_template').className = document.getElementById('odt_template').className.replace('row_visible', 'row_hidden');
	   document.getElementById('xslt_template').className = document.getElementById('xslt_template').className.replace('row_visible', 'row_hidden');
	   document.getElementById('absences').className = document.getElementById('absences').className.replace('row_visible', 'row_hidden');
	\};
\}
</script>
	<div id=\"fullwidth-list\" class=\"fullwidth-list\">
     [im_box_header $page_title_adjusted]
     <span id='chart_pie'></span> 
"

switch $output_format {

    html {
	   ns_write $sidebar_html
	   ns_write "<table border=0 cellspacing='2' cellpadding='2' class='table_list_simple'>\n"
    }

    printer {
	ns_write "
	<link rel=StyleSheet type='text/css' href='/intranet-reporting/printer-friendly.css' media=all>
        <div class=\"fullwidth-list\">
	<table border='0' cellspacing='1' cellpadding='1' rules='all'>
	<colgroup>
		<col id=datecol>
		<col id=hourcol>
		<col id=datecol>
		<col id=datecol>
		<col id=hourcol>
		<col id=hourcol>
		<col id=hourcol>
	</colgroup>
	"
    }
	xml - template {
		set xml_doc [dom parse "<report></report>"]
		set root [$xml_doc documentElement]
		set node [$root selectNodes "/report"]
		$node appendFromList [list attributes {} {}]
		set node [$root selectNodes "/report/attributes"]
		$node appendFromList [list start_date [list] [list [list "\#text" [encodeXmlValue $start_date]]]]		
		$node appendFromList [list end_date [list] [list [list "\#text" [encodeXmlValue $end_date]]]]
		$node appendFromList [list filter_user_id [list] [list [list "\#text" $user_id]]]		
		$node appendFromList [list filter_company_id [list] [list [list "\#text" $company_id]]]		
		$node appendFromList [list filter_project_id [list] [list [list "\#text" $project_id]]]		
		$node appendFromList [list name_report_creator [list] [list [list "\#text" [encodeXmlValue [im_name_from_user_id $current_user_id]]]]]

		if { (0 != $company_id && "" != $company_id) || (0 != $project_id && "" != $project_id)} {
		   	 if { 0 == $company_id || "" == $company_id } {
			 	# Get company_id from project 
				set company_id_sql [db_string get_company_id "select company_id from im_projects where project_id = :project_id" -default 0]
			 } else {
			   	set company_id_sql $company_id
			 }
			 db_1row company_info "
					select
						c.company_name as company_name,
						c.vat_number as company_vat_number,
						im_name_from_user_id(c.manager_id) as company_manager_name,
						im_email_from_user_id(c.manager_id) as company_manager_email,
						im_name_from_user_id(c.primary_contact_id) as company_primary_contact_name,
						im_email_from_user_id(c.primary_contact_id) as company_primary_contact_email,
						im_name_from_user_id(c.accounting_contact_id) as company_accounting_contact_name,
						im_email_from_user_id(c.accounting_contact_id) as company_accounting_contact_email,
						o.office_name as company_office_name,
						o.fax as company_fax,
						o.phone as company_phone,
						o.address_line1 as company_address_line1,
						o.address_line2 as company_address_line2,
						o.address_city as company_city,
						o.address_state as company_state,
						o.address_postal_code as company_postal_code,
						cou.country_name as company_country_name
					from
						im_companies c
						LEFT OUTER JOIN im_offices o ON (c.main_office_id = o.office_id)
						LEFT OUTER JOIN country_codes cou ON (o.address_country_code = iso)
						LEFT OUTER JOIN im_categories paymeth ON (c.default_payment_method_id = paymeth.category_id)
				    where
					    c.company_id = :company_id_sql
			"
	        set node [$root selectNodes "/report"]
			$node appendFromList [list company {} {}]
			set node [$root selectNodes "/report/company"]			
			$node appendFromList [list company_name [list] [list [list "\#text" [encodeXmlValue $company_name]]]]
			$node appendFromList [list company_vat_number [list] [list [list "\#text" [encodeXmlValue $company_vat_number]]]]
			$node appendFromList [list company_manager_name [list] [list [list "\#text" [encodeXmlValue $company_manager_name]]]]
			$node appendFromList [list company_manager_email [list] [list [list "\#text" [encodeXmlValue $company_manager_email]]]]
			$node appendFromList [list company_primary_contact_name [list] [list [list "\#text" [encodeXmlValue $company_primary_contact_name]]]]
			$node appendFromList [list company_primary_contact_email [list] [list [list "\#text" [encodeXmlValue $company_primary_contact_email]]]]
			$node appendFromList [list company_accounting_contact_name [list] [list [list "\#text" [encodeXmlValue $company_accounting_contact_name]]]]
			$node appendFromList [list company_accounting_contact_email [list] [list [list "\#text" [encodeXmlValue $company_accounting_contact_email]]]]
			$node appendFromList [list company_office_name [list] [list [list "\#text" [encodeXmlValue $company_office_name]]]]
			$node appendFromList [list company_fax [list] [list [list "\#text" [encodeXmlValue $company_fax]]]]
			$node appendFromList [list company_phone [list] [list [list "\#text" [encodeXmlValue $company_phone]]]]
			$node appendFromList [list company_address_line1 [list] [list [list "\#text" [encodeXmlValue $company_address_line1]]]]
			$node appendFromList [list company_address_line2 [list] [list [list "\#text" [encodeXmlValue $company_address_line2]]]]
			$node appendFromList [list company_city [list] [list [list "\#text" [encodeXmlValue $company_city]]]]
			$node appendFromList [list company_state [list] [list [list "\#text" [encodeXmlValue $company_state]]]]
			$node appendFromList [list company_postal_code [list] [list [list "\#text" [encodeXmlValue $company_postal_code]]]]
			$node appendFromList [list company_country_name [list] [list [list "\#text" [encodeXmlValue $company_country_name]]]]
		}

        if { 0 != $project_id && "" != $project_id} {
			set project_name [db_string get_project_name "select project_name from im_projects where project_id = :project_id" -default 0] 
			set node [$root selectNodes "/report"]
			$node appendFromList [list project {} {}]
			set node [$root selectNodes "/report/project"]
			$node appendFromList [list project_name [list] [list [list "\#text" [encodeXmlValue $project_name]]]]
		}

        if { 0 != $user_id && "" != $user_id } {
			set user_name [im_name_from_user_id $user_id] 
			set node [$root selectNodes "/report"]
			$node appendFromList [list user {} {}]
			set node [$root selectNodes "/report/user"]
			$node appendFromList [list user_name [list] [list [list "\#text" [encodeXmlValue $user_name]]]]
		}
		   
		# Add node 'lines' 
        set node [$root selectNodes "/report"]
        $node appendFromList [list lines [list] [list]]
		#ns_return 1 text/xml [$root asXML]
    }
}

im_report_render_row \
    -output_format $output_format \
    -row $header0 \
    -row_class "rowtitle" \
    -cell_class "rowtitle"


set footer_array_list [list]
set last_value_list [list]
set class "rowodd"

db_foreach sql $sql {

	# Does the user prefer to read project_name instead of project_nr?
	if {$use_project_name_p} { 
	    set project_nr $project_name
	    set sub_project_name [im_reporting_sub_project_name_path $sub_project_id]
	    set sub_project_nr $sub_project_name
	    set user_initials $user_name
	    set company_nr $company_name
	}

	if {"" != $internal_note} {
	    set note "$note / $internal_note"
	}
	if {[string length $note] > $truncate_note_length} {
	    set note "[string range $note 0 $truncate_note_length] ..."
	}
	set hours_link $hours
	if {$edit_timesheet_p} {
	    set hours_link " <a href=\"[export_vars -base $hours_url {julian_date user_id {project_id $sub_project_id} {return_url $this_url}}]\">$hours</a>\n"
	}

	im_report_display_footer \
	    -output_format $output_format \
	    -group_def $report_def \
	    -footer_array_list $footer_array_list \
	    -last_value_array_list $last_value_list \
	    -level_of_detail $level_of_detail \
	    -row_class $class \
	    -cell_class $class

	im_report_update_counters -counters $counters
	
	set last_value_list [im_report_render_header \
	    -output_format $output_format \
	    -group_def $report_def \
	    -last_value_array_list $last_value_list \
	    -level_of_detail $level_of_detail \
	    -row_class $class \
	    -cell_class $class
	]

	set footer_array_list [im_report_render_footer \
	    -output_format $output_format \
	    -group_def $report_def \
	    -last_value_array_list $last_value_list \
	    -level_of_detail $level_of_detail \
	    -row_class $class \
	    -cell_class $class
	]

	if { "xml" == $output_format || "template" == $output_format } {

		set attributes "company_id='$company_id'"
		append attributes " project_id ='$project_id'" 
		append attributes " project_name ='[encodeXmlValue $project_name]'" 
		append attributes " project_nr = '[encodeXmlValue $project_nr]'" 
		append attributes " sub_project_id= '$sub_project_id'"
 		append attributes " sub_project_name = '[encodeXmlValue $sub_project_name]'" 
		append attributes " sub_project_nr ='[encodeXmlValue $sub_project_nr]'" 
		append attributes " user_id = '$user_id'" 
		append attributes " user_name = '[encodeXmlValue $user_name]'" 
		append attributes " day ='$date_pretty'"
		append attributes " hours ='$hours'"
		append attributes " note ='[encodeXmlValue $note]'" 
		append attributes " internal_note ='[encodeXmlValue $internal_note]'"  

		set node [$root selectNodes "/report/lines"]
		$node appendXML "<rec $attributes />"

	}
}

#-- ------------------------------------------------------------------------------------
#   Creating absences 
#-- ------------------------------------------------------------------------------------

foreach user_absence $absence_list {

   #     $hours_user_counter 
   #     $hours_project_sub_counter 
   #     $hours_project_counter 
   #     $hours_customer_counter 
   # 

    set k [lindex $user_absence 0]
    set v [lindex $user_absence 1]
    
    # ds_comment "$k / $v"  
    # ds_comment "company_project_sub_id: $company_project_sub_id"

    # 0=$hours, 1=Absence Type, 2=Date, 3=user_id   

    set note ""
    set internal_note ""
    set hour_id 0 
    set date_pretty [lindex $v 2]
    set julian_date "0"
    set date_diff ""
    set hours [lindex $v 0]
    set hours_link $hours
    set billing_rate 0
    set user_id [lindex $v 3]
    set user_name [im_name_from_user_id [lindex $v 3]]
    set user_initials [lindex $v 4]
    set project_id 0
    set project_nr ""
    set project_name "Absences"
    set sub_project_id [lindex $v 1]
    set sub_project_nr [im_category_from_id [lindex $v 1]]
    set sub_project_name [im_category_from_id [lindex $v 1]]
    set company_id 0
    set company_nr ""
    set company_name "Company"
    set company_project_id 99
    set company_project_sub_id [lindex $v 1] 
    set company_project_sub_user_id [lindex $v 3]

    im_report_display_footer \
         -output_format $output_format \
         -group_def $report_def \
         -footer_array_list $footer_array_list \
         -last_value_array_list $last_value_list \
         -level_of_detail $level_of_detail \
         -row_class $class \
         -cell_class $class

    im_report_update_counters -counters $counters

    set last_value_list [im_report_render_header \
         -output_format $output_format \
         -group_def $report_def \
         -last_value_array_list $last_value_list \
         -level_of_detail $level_of_detail \
         -row_class $class \
         -cell_class $class
    ]

    set footer_array_list [im_report_render_footer \
         -output_format $output_format \
         -group_def $report_def \
         -last_value_array_list $last_value_list \
         -level_of_detail $level_of_detail \
         -row_class $class \
         -cell_class $class
    ]
}


im_report_display_footer \
    -output_format $output_format \
    -group_def $report_def \
    -footer_array_list $footer_array_list \
    -last_value_array_list $last_value_list \
    -level_of_detail $level_of_detail \
    -display_all_footers_p 1 \
    -row_class $class \
    -cell_class $class

im_report_render_row \
    -output_format $output_format \
    -row $footer0 \
    -row_class $class \
    -cell_class $class


# Write out the HTMl to close the main report table
# and write out the page footer.

switch $output_format {
    html { 
	ns_write "</table>[im_box_footer]</div></form>"
	# Todo: Verify how to include div id "monitor_frame" to make following js obsolete 
	ns_write "<script language='javascript' type='text/javascript' nonce='[im_csp_nonce]'>document.getElementById('slave_content').style.visibility='visible';"
	ns_write "document.getElementById('fullwidth-list').style.visibility='visible'; </script>"
	ns_write "[im_footer]\n"
    }
    printer { ns_write "</table>\n</div>\n"}
    csv { }
    xml {
        if { "" != $xslt_template_id  } {
            set uri_xslt "$invoice_template_base_path/[im_category_from_id $xslt_template_id]"
            ns_log Notice "timesheet-customer-project-v2 :: uri_xslt: $uri_xslt"

	    # Create tmp file for Reports Default XML
	    set uri_report_default_xml [ad_tmpnam]
	    set fo [open $uri_report_default_xml {RDWR CREAT}]
	    puts $fo "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>[$root asXML]"
	    close $fo
	    
            # Create tmp file for Report Custom XML
            set uri_report_custom_xml [ad_tmpnam]
            set fo [open $uri_report_custom_xml {RDWR CREAT}]
            puts $fo " "
            close $fo

            # Perform transition
            im_exec java -jar $path_saxon -s:$uri_report_default_xml -xsl:$uri_xslt -o:$uri_report_custom_xml 
	    ns_returnfile 200 text/xml $uri_report_custom_xml

	} else {
            # In case no transition file has been assigned we return the default XML file
	    ns_return 200 text/xml [$root asXML]
        }
    }
    template {
	# Create tmp file for Reports Default XML 
	set uri_report_default_xml [ad_tmpnam]
	set fo [open $uri_report_default_xml {RDWR CREAT}]
	puts $fo "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>[$root asXML]"
	close $fo		
	
	# Get URI of XSLT choosen by user  
	if { "" != $xslt_template_id  } {
	    set uri_xslt "$invoice_template_base_path/[im_category_from_id $xslt_template_id]"
	    ns_log Notice "timesheet-customer-project-v2 :: uri_xslt: $uri_xslt"
	    
	    # Create tmp file for Report Custom XML 
	    set uri_report_custom_xml [ad_tmpnam]
	    set fo [open $uri_report_custom_xml {RDWR CREAT}]
	    puts $fo " "
	    close $fo
	    im_exec java -jar $path_saxon -s:$uri_report_default_xml -xsl:$uri_xslt -o:${uri_report_custom_xml}
	} else {
	    # In case no transition file has been assigned we continue working with the default XML file 
	    set uri_report_custom_xml $uri_report_default_xml
	}
	
	ns_log Notice "timesheet-customer-project-v2 :: uri_report_xml: $uri_report_custom_xml"
	
	# Render Template
	set odt_zip [im_reporting_render_odt_template "${invoice_template_base_path}/[im_category_from_id $odt_template_id]" $uri_report_custom_xml ""]
	
	# Return file 
	set outputheaders [ns_conn outputheaders]
	set suffix [lindex [split $odt_zip "." ] [expr [llength [split $odt_zip "." ]]-1]]
	ns_set cput $outputheaders "Content-Disposition" "attachment; filename=timesheet-customer-project.$suffix"
	ns_returnfile 200 application/odt $odt_zip
    }
    chart {
	# Todo: Verify how to include div id "monitor_frame" to make following js obsolete 
	ns_write "<script language='javascript' type='text/javascript' nonce='[im_csp_nonce]'>document.getElementById('slave_content').style.visibility='visible';"
	ns_write "document.getElementById('fullwidth-list').style.visibility='visible'; </script>"
	# These var values we need to wrap into ''
	set str_vars_list [list]
	lappend str_vars_list start_date
	lappend str_vars_list end_date
        lappend str_vars_list number_format

	# replace colon vars with their values
	split $sql " "
	foreach sub_str $sql {
	    if { -1 != [string first : $sub_str] } {
		if { -1 == [string first :: $sub_str] } {
		    set var_name [string range $sub_str 1 end]  
		    eval set str_r $$var_name
		    if { -1 == [lsearch -glob $str_vars_list $var_name] } {
			lappend parsed_sql $str_r 
		    } else {
			lappend parsed_sql "'$str_r'" 
		    }	   
		} else {
		    lappend parsed_sql $sub_str
		}
	    } else {
		lappend parsed_sql $sub_str
	    }
	}
	
	set parsed_sql [join $parsed_sql " "]
	
	# set rand_key [ad_generate_random_string 256]
	# set sql_rep "
	#	   insert into im_cached_reports (user_id, rand_key, report_label, request_timestamp, report_sql) values (:current_user_id, :rand_key, 'reporting-timesheet-customer-project-xml-xslt', now(), :parsed_sql)
	# "	   
	# set query_id [db_dml insert_report $sql_rep]
	
	ns_write $sidebar_html
        ns_write "[im_box_footer]</div></form>"

	   # Call template to generate JS that creates chart 
	   # set params [list [list current_user_id $current_user_id] [list query_id $query_id] [list rand_key $rand_key] [list chart_type $chart_type] [list mode del] [sql $parsed_sql]]
	   set params [list [list current_user_id $current_user_id] [list chart_type $chart_type] [list mode del] [list sql $parsed_sql] [list chart_type $chart_type]]
	   ns_write [ad_parse_template -params $params "/packages/intranet-reporting/lib/create-ts-chart"]
        ns_write "[im_footer]\n"

        # Todo: Verify how to include div id "monitor_frame" to make following js obsolete
        ns_write "<script language='javascript' type='text/javascript' nonce='[im_csp_nonce]'>"
        ns_write "document.getElementById('slave_content').style.visibility='visible';"
        ns_write "document.getElementById('fullwidth-list').style.visibility='visible';" 
        ns_write "</script>"
    }

    default {
	ad_return_complaint 1 "Error: No template defined"
    }
}
